/*  Note: You are free to use whatever license you want.
    Eventually you will be able to edit it within Glade. */

/*  gxhost
 *  Copyright (C) <YEAR> <AUTHORS>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "gxhostsig.h"
#include "gxhostsrc.h"

#include "gxhost.xpm"

GtkWidget*
get_widget                             (GtkWidget       *widget,
                                        gchar           *widget_name)
{
  GtkWidget *parent, *found_widget;

  for (;;)
    {
      if (GTK_IS_MENU (widget))
        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
      else
        parent = widget->parent;
      if (parent == NULL)
        break;
      widget = parent;
    }

  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to set notebook tab widgets. */
void
set_notebook_tab                       (GtkWidget       *notebook,
                                        gint             page_num,
                                        GtkWidget       *widget)
{
  GtkNotebookPage *page;
  GtkWidget *notebook_page;

  page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
  notebook_page = page->child;
  gtk_widget_ref (notebook_page);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
  gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
                            widget, page_num);
  gtk_widget_unref (notebook_page);
}

static GList *pixmaps_directories = NULL;

/* Use this function to set the directory containing installed pixmaps. */
void
add_pixmap_directory                   (gchar           *directory)
{
  pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
}

/* This is an internally used function to check if a pixmap file exists. */
#ifndef G_DIR_SEPARATOR_S
#define G_DIR_SEPARATOR_S "/"
#endif
gchar*
check_file_exists                      (gchar           *directory,
                                        gchar           *filename)
{
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = g_malloc (strlen (directory) + 1 + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/* This is an internally used function to create pixmaps. */
GtkWidget*
create_pixmap                          (GtkWidget       *widget,
                                        gchar           *filename)
{
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  GList *elem;

  /* We first try any pixmaps directories set by the application. */
  elem = pixmaps_directories;
  while (elem)
    {
      found_filename = check_file_exists ((gchar*)elem->data, filename);
      if (found_filename)
        break;
      elem = elem->next;
    }

  /* If we haven't found the pixmap, try the source directory. */
  if (!found_filename)
    {
      found_filename = check_file_exists ("../xtra/xpm", filename);
    }

  if (!found_filename)
    {
      g_print ("Couldn't find pixmap file: %s", filename);
      return NULL;
    }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
                                                   NULL, found_filename);
  g_free (found_filename);
  if (gdkpixmap == NULL)
    return NULL;
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

GtkWidget*
create_win_main ()
{
  GtkWidget *win_main;
  GtkWidget *vbox1;
  GtkWidget *handlebox1;
  GtkWidget *mnu_main;
  GtkWidget *file;
  GtkWidget *file_menu;
  GtkWidget *quit;
  GtkWidget *xhost;
  GtkWidget *xhost_menu;
  GtkWidget *allow_all;
  GtkWidget *deny_all;
  GtkWidget *separator1;
  GtkWidget *allow_current;
  GtkWidget *deny_current;
  GtkWidget *separator2;
  GtkWidget *refresh;
  GtkWidget *options;
  GtkWidget *options_menu;
  GtkWidget *hide_Toolbar;
  GtkWidget *helpm;
  GtkWidget *helpm_menu;
  GtkWidget *help;
  GtkWidget *about;
  GtkWidget *handlebox2;
  GtkWidget *toolbar2;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *btn_allowall;
  GtkWidget *btn_denyall;
  GtkWidget *btn_refresh;
  GtkWidget *btn_help;
  GtkWidget *btn_exit;
  GtkWidget *frame2;
  GtkWidget *hbox2;
  GtkWidget *cmb_host;
  GtkWidget *btn_allow;
  GtkWidget *pixmap1;
  GtkWidget *frame1;
  GtkWidget *hbox3;
  GtkWidget *scrolledwindow1;
  GtkWidget *ctr_hosts;
  GtkWidget *label5;
  GtkWidget *label6;
  GtkWidget *vbuttonbox1;
  GtkWidget *btn_deny;
  GtkWidget *pixmap2;
  GtkWidget *sbr_main;
  GtkAccelGroup *accel_group;
  GtkTooltips *tooltips;
  GdkPixmap *icon_pixmap;
  GdkBitmap *icon_mask;      

  tooltips = gtk_tooltips_new ();

  win_main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (win_main), "win_main", win_main);
  gtk_widget_set_usize (win_main, 320, 300);
  gtk_signal_connect (GTK_OBJECT (win_main), "delete_event",
                      GTK_SIGNAL_FUNC (on_win_main_delete_event),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (win_main), "gxhost");
  gtk_window_set_policy (GTK_WINDOW (win_main), TRUE, TRUE, FALSE);

     gtk_widget_realize (win_main);
     icon_pixmap = gdk_pixmap_create_from_xpm_d (win_main->window, &icon_mask,
                               &win_main->style->bg[GTK_STATE_NORMAL],gxhost_xpm);
     gdk_window_set_icon (win_main->window, NULL, icon_pixmap, icon_mask);
                          
                                           
     gdk_window_set_icon_name(win_main->window,"gxhost");                                          


  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (win_main), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (win_main), vbox1);

  handlebox1 = gtk_handle_box_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "handlebox1", handlebox1);
  gtk_widget_show (handlebox1);
  gtk_box_pack_start (GTK_BOX (vbox1), handlebox1, FALSE, TRUE, 0);

  mnu_main = gtk_menu_bar_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "mnu_main", mnu_main);
  gtk_widget_show (mnu_main);
  gtk_container_add (GTK_CONTAINER (handlebox1), mnu_main);

  file = gtk_menu_item_new_with_label ("File");
  gtk_object_set_data (GTK_OBJECT (win_main), "file", file);
  gtk_widget_show (file);
  gtk_container_add (GTK_CONTAINER (mnu_main), file);

  file_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "file_menu", file_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file), file_menu);

  quit = gtk_menu_item_new_with_label ("Quit");
  gtk_object_set_data (GTK_OBJECT (win_main), "quit", quit);
  gtk_widget_show (quit);
  gtk_container_add (GTK_CONTAINER (file_menu), quit);
  gtk_signal_connect (GTK_OBJECT (quit), "activate",
                      GTK_SIGNAL_FUNC (on_quit_activate),
                      NULL);
  accel_group = gtk_accel_group_new ();
  gtk_window_add_accel_group (GTK_WINDOW (win_main), accel_group);
  gtk_widget_add_accelerator (quit, "activate", accel_group,
                              GDK_Q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  xhost = gtk_menu_item_new_with_label ("xhost");
  gtk_object_set_data (GTK_OBJECT (win_main), "xhost", xhost);
  gtk_widget_show (xhost);
  gtk_container_add (GTK_CONTAINER (mnu_main), xhost);

  xhost_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "xhost_menu", xhost_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (xhost), xhost_menu);

  allow_all = gtk_menu_item_new_with_label ("Allow all");
  gtk_object_set_data (GTK_OBJECT (win_main), "allow_all", allow_all);
  gtk_widget_show (allow_all);
  gtk_container_add (GTK_CONTAINER (xhost_menu), allow_all);
  gtk_signal_connect (GTK_OBJECT (allow_all), "activate",
                      GTK_SIGNAL_FUNC (on_allow_all_activate),
                      NULL);
  gtk_widget_add_accelerator (allow_all, "activate", accel_group,
                              GDK_A, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  deny_all = gtk_menu_item_new_with_label ("Deny all");
  gtk_object_set_data (GTK_OBJECT (win_main), "deny_all", deny_all);
  gtk_widget_show (deny_all);
  gtk_container_add (GTK_CONTAINER (xhost_menu), deny_all);
  gtk_signal_connect (GTK_OBJECT (deny_all), "activate",
                      GTK_SIGNAL_FUNC (on_deny_all_activate),
                      NULL);
  gtk_widget_add_accelerator (deny_all, "activate", accel_group,
                              GDK_D, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  separator1 = gtk_menu_item_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "separator1", separator1);
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (xhost_menu), separator1);

  allow_current = gtk_menu_item_new_with_label ("Allow current");
  gtk_object_set_data (GTK_OBJECT (win_main), "allow_current", allow_current);
  gtk_widget_show (allow_current);
  gtk_container_add (GTK_CONTAINER (xhost_menu), allow_current);
  gtk_signal_connect (GTK_OBJECT (allow_current), "activate",
                      GTK_SIGNAL_FUNC (on_allow_current_activate),
                      NULL);

  deny_current = gtk_menu_item_new_with_label ("Deny current");
  gtk_object_set_data (GTK_OBJECT (win_main), "deny_current", deny_current);
  gtk_widget_show (deny_current);
  gtk_container_add (GTK_CONTAINER (xhost_menu), deny_current);
  gtk_signal_connect (GTK_OBJECT (deny_current), "activate",
                      GTK_SIGNAL_FUNC (on_deny_current_activate),
                      NULL);

  separator2 = gtk_menu_item_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "separator2", separator2);
  gtk_widget_show (separator2);
  gtk_container_add (GTK_CONTAINER (xhost_menu), separator2);

  refresh = gtk_menu_item_new_with_label ("Refresh");
  gtk_object_set_data (GTK_OBJECT (win_main), "refresh", refresh);
  gtk_widget_show (refresh);
  gtk_container_add (GTK_CONTAINER (xhost_menu), refresh);
  gtk_signal_connect (GTK_OBJECT (refresh), "activate",
                      GTK_SIGNAL_FUNC (on_refresh_activate),
                      NULL);
  gtk_widget_add_accelerator (refresh, "activate", accel_group,
                              GDK_R, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  options = gtk_menu_item_new_with_label ("Options");
  gtk_object_set_data (GTK_OBJECT (win_main), "options", options);
  gtk_widget_show (options);
  gtk_container_add (GTK_CONTAINER (mnu_main), options);

  options_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "options_menu", options_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (options), options_menu);

  hide_Toolbar = gtk_check_menu_item_new_with_label ("Hide Toolbar");
  gtk_object_set_data (GTK_OBJECT (win_main), "hide_Toolbar", hide_Toolbar);
  gtk_widget_show (hide_Toolbar);
  gtk_container_add (GTK_CONTAINER (options_menu), hide_Toolbar);
  gtk_signal_connect (GTK_OBJECT (hide_Toolbar), "activate",
                      GTK_SIGNAL_FUNC (on_hide_Toolbar_activate),
                      NULL);

  helpm = gtk_menu_item_new_with_label ("Help");
  gtk_object_set_data (GTK_OBJECT (win_main), "helpm", helpm);
  gtk_widget_show (helpm);
  gtk_container_add (GTK_CONTAINER (mnu_main), helpm);
  gtk_signal_connect (GTK_OBJECT (helpm), "activate",
                      GTK_SIGNAL_FUNC (on_helpm_activate),
                      NULL);
  gtk_menu_item_right_justify (GTK_MENU_ITEM (helpm));

  helpm_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "helpm_menu", helpm_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (helpm), helpm_menu);

  help = gtk_menu_item_new_with_label ("Help");
  gtk_object_set_data (GTK_OBJECT (win_main), "help", help);
  gtk_widget_show (help);
  gtk_container_add (GTK_CONTAINER (helpm_menu), help);
  gtk_signal_connect (GTK_OBJECT (help), "activate",
                      GTK_SIGNAL_FUNC (on_help_activate),
                      NULL);
  gtk_widget_add_accelerator (help, "activate", accel_group,
                              GDK_H, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  about = gtk_menu_item_new_with_label ("About");
  gtk_object_set_data (GTK_OBJECT (win_main), "about", about);
  gtk_widget_show (about);
  gtk_container_add (GTK_CONTAINER (helpm_menu), about);
  gtk_signal_connect (GTK_OBJECT (about), "activate",
                      GTK_SIGNAL_FUNC (on_about_activate),
                      NULL);

  handlebox2 = gtk_handle_box_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "handlebox2", handlebox2);
  gtk_widget_show (handlebox2);
  gtk_box_pack_start (GTK_BOX (vbox1), handlebox2, FALSE, TRUE, 0);

  toolbar2 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_object_set_data (GTK_OBJECT (win_main), "toolbar2", toolbar2);
  gtk_widget_show (toolbar2);
  gtk_container_add (GTK_CONTAINER (handlebox2), toolbar2);

  tmp_toolbar_icon = create_pixmap (win_main, "allow.xpm");
  if (tmp_toolbar_icon == NULL)
    g_error ("Couldn't create pixmap");
  btn_allowall = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Allow",
                                "Allow any host", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (win_main), "btn_allowall", btn_allowall);
  gtk_widget_show (btn_allowall);
  gtk_signal_connect (GTK_OBJECT (btn_allowall), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_allowall_clicked),
                      NULL);
  gtk_widget_add_accelerator (btn_allowall, "clicked", accel_group,
                              GDK_A, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  tmp_toolbar_icon = create_pixmap (win_main, "deny.xpm");
  if (tmp_toolbar_icon == NULL)
    g_error ("Couldn't create pixmap");
  btn_denyall = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Deny",
                                "Deny all hosts", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (win_main), "btn_denyall", btn_denyall);
  gtk_widget_show (btn_denyall);
  gtk_signal_connect (GTK_OBJECT (btn_denyall), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_denyall_clicked),
                      NULL);
  gtk_widget_add_accelerator (btn_denyall, "clicked", accel_group,
                              GDK_D, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  tmp_toolbar_icon = create_pixmap (win_main, "refresh.xpm");
  if (tmp_toolbar_icon == NULL)
    g_error ("Couldn't create pixmap");
  btn_refresh = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Refresh",
                                "Refresh allowed hosts", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (win_main), "btn_refresh", btn_refresh);
  gtk_widget_show (btn_refresh);
  gtk_signal_connect (GTK_OBJECT (btn_refresh), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_refresh_clicked),
                      NULL);
  gtk_widget_add_accelerator (btn_refresh, "clicked", accel_group,
                              GDK_R, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  tmp_toolbar_icon = create_pixmap (win_main, "help.xpm");
  if (tmp_toolbar_icon == NULL)
    g_error ("Couldn't create pixmap");
  btn_help = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Help",
                                "Help", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (win_main), "btn_help", btn_help);
  gtk_widget_show (btn_help);
  gtk_signal_connect (GTK_OBJECT (btn_help), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_help_clicked),
                      NULL);
  gtk_widget_add_accelerator (btn_help, "clicked", accel_group,
                              GDK_H, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  tmp_toolbar_icon = create_pixmap (win_main, "exit.xpm");
  if (tmp_toolbar_icon == NULL)
    g_error ("Couldn't create pixmap");
  btn_exit = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Exit",
                                "Quit this program", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (win_main), "btn_exit", btn_exit);
  gtk_widget_show (btn_exit);
  gtk_signal_connect (GTK_OBJECT (btn_exit), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_exit_clicked),
                      NULL);
  gtk_widget_add_accelerator (btn_exit, "clicked", accel_group,
                              GDK_Q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator (btn_exit, "clicked", accel_group,
                              GDK_X, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);

  frame2 = gtk_frame_new ("Allow a new host");
  gtk_object_set_data (GTK_OBJECT (win_main), "frame2", frame2);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, FALSE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (frame2), 6);

  hbox2 = gtk_hbox_new (FALSE, 4);
  gtk_object_set_data (GTK_OBJECT (win_main), "hbox2", hbox2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (frame2), hbox2);

  cmb_host = gtk_combo_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "cmb_host", cmb_host);
  gtk_widget_show (cmb_host);
  gtk_box_pack_start (GTK_BOX (hbox2), cmb_host, TRUE, TRUE, 0);

  btn_allow = gtk_button_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "btn_allow", btn_allow);
  gtk_widget_show (btn_allow);
  gtk_box_pack_start (GTK_BOX (hbox2), btn_allow, FALSE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, btn_allow, "Allow this host", NULL);
  gtk_signal_connect (GTK_OBJECT (btn_allow), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_allow_clicked),
                      NULL);
  gtk_widget_add_accelerator (btn_allow, "clicked", accel_group,
                              GDK_plus, 0, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator (btn_allow, "clicked", accel_group,
                              GDK_KP_Add, 0, GTK_ACCEL_VISIBLE);

  pixmap1 = create_pixmap (win_main, "allow.xpm");
  if (pixmap1 == NULL)
    g_error ("Couldn't create pixmap");
  gtk_object_set_data (GTK_OBJECT (win_main), "pixmap1", pixmap1);
  gtk_widget_show (pixmap1);
  gtk_container_add (GTK_CONTAINER (btn_allow), pixmap1);

  frame1 = gtk_frame_new ("Current allowed hosts");
  gtk_object_set_data (GTK_OBJECT (win_main), "frame1", frame1);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (frame1), 6);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (win_main), "hbox3", hbox3);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (frame1), hbox3);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (win_main), "scrolledwindow1", scrolledwindow1);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (hbox3), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  ctr_hosts = gtk_ctree_new (2, 0);
  gtk_object_set_data (GTK_OBJECT (win_main), "ctr_hosts", ctr_hosts);
  gtk_widget_show (ctr_hosts);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), ctr_hosts);
  gtk_signal_connect (GTK_OBJECT (ctr_hosts), "tree_select_row",
                      GTK_SIGNAL_FUNC (on_ctr_hosts_tree_select_row),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctr_hosts), "tree_unselect_row",
                      GTK_SIGNAL_FUNC (on_ctr_hosts_tree_unselect_row),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (ctr_hosts), "key_release_event",
                      GTK_SIGNAL_FUNC (on_ctr_hosts_key_press_event),
                      NULL);
  gtk_clist_set_column_width (GTK_CLIST (ctr_hosts), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (ctr_hosts), 1, 80);
  gtk_clist_column_titles_show (GTK_CLIST (ctr_hosts));

  label5 = gtk_label_new ("Family");
  gtk_object_set_data (GTK_OBJECT (win_main), "label5", label5);
  gtk_widget_show (label5);
  gtk_clist_set_column_widget (GTK_CLIST (ctr_hosts), 0, label5);

  label6 = gtk_label_new ("Hosts");
  gtk_object_set_data (GTK_OBJECT (win_main), "label6", label6);
  gtk_widget_show (label6);
  gtk_clist_set_column_widget (GTK_CLIST (ctr_hosts), 1, label6);

  vbuttonbox1 = gtk_vbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "vbuttonbox1", vbuttonbox1);
  gtk_widget_show (vbuttonbox1);
  gtk_box_pack_start (GTK_BOX (hbox3), vbuttonbox1, FALSE, TRUE, 0);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (vbuttonbox1), 48, 27);

  btn_deny = gtk_button_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "btn_deny", btn_deny);
  gtk_widget_show (btn_deny);
  gtk_container_add (GTK_CONTAINER (vbuttonbox1), btn_deny);
  gtk_container_border_width (GTK_CONTAINER (btn_deny), 2);
  gtk_tooltips_set_tip (tooltips, btn_deny, "Deny current selected host", NULL);
  gtk_signal_connect (GTK_OBJECT (btn_deny), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_deny_clicked),
                      NULL);
  gtk_widget_add_accelerator (btn_deny, "clicked", accel_group,
                              GDK_minus, 0, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator (btn_deny, "clicked", accel_group,
                              GDK_KP_Subtract, 0, GTK_ACCEL_VISIBLE);

  pixmap2 = create_pixmap (win_main, "deny.xpm");
  if (pixmap2 == NULL)
    g_error ("Couldn't create pixmap");
  gtk_object_set_data (GTK_OBJECT (win_main), "pixmap2", pixmap2);
  gtk_widget_show (pixmap2);
  gtk_container_add (GTK_CONTAINER (btn_deny), pixmap2);

  sbr_main = gtk_statusbar_new ();
  gtk_object_set_data (GTK_OBJECT (win_main), "sbr_main", sbr_main);
  gtk_widget_show (sbr_main);
  gtk_box_pack_start (GTK_BOX (vbox1), sbr_main, FALSE, TRUE, 0);

  gtk_object_set_data (GTK_OBJECT (win_main), "tooltips", tooltips);

  return win_main;
}

GtkWidget*
create_win_msg (char *title,char *msg)
{
  GtkWidget *win_msg;
  GtkWidget *vbox6;
  GtkWidget *lbl_msg;
  GtkWidget *hbuttonbox7;
  GtkWidget *btn_msgok;

  win_msg = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (win_msg), "win_msg", win_msg);
  gtk_window_set_title (GTK_WINDOW (win_msg), title);
  gtk_window_set_policy (GTK_WINDOW (win_msg), TRUE, TRUE, FALSE);

  vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (win_msg), "vbox6", vbox6);
  gtk_widget_show (vbox6);
  gtk_container_add (GTK_CONTAINER (win_msg), vbox6);

  lbl_msg = gtk_label_new (msg);
  gtk_object_set_data (GTK_OBJECT (win_msg), "lbl_msg", lbl_msg);
  gtk_widget_show (lbl_msg);
  gtk_box_pack_start (GTK_BOX (vbox6), lbl_msg, TRUE, TRUE, 0);

  hbuttonbox7 = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (win_msg), "hbuttonbox7", hbuttonbox7);
  gtk_widget_show (hbuttonbox7);
  gtk_box_pack_start (GTK_BOX (vbox6), hbuttonbox7, FALSE, TRUE, 3);

  btn_msgok = gtk_button_new_with_label ("Dismiss");
  gtk_object_set_data (GTK_OBJECT (win_msg), "btn_msgok", btn_msgok);
  gtk_widget_show (btn_msgok);
  gtk_container_add (GTK_CONTAINER (hbuttonbox7), btn_msgok);
  gtk_signal_connect (GTK_OBJECT (btn_msgok), "clicked",
                      GTK_SIGNAL_FUNC (on_btn_msgok_clicked),
                      NULL);

  return win_msg;
}



